@@ -17,6 +17,7 @@ int update_utmp(const char *login_fmt, const char *from_fmt, const struct auth *
 	FILE *fp;
 	pid_t pid;
 	char tmp_str [256];
+      	char *tty;
 
 	if(!lineconf.sysutmp)
 		return 0;
@@ -34,29 +35,33 @@ int update_utmp(const char *login_fmt, const char *from_fmt, const struct auth *
 		}
 	}
 	if (utmptr) {
-		char *tty = lineconf.tty;
-		if(!strncmp(tty, "/dev/", 5))
-			tty += 5;
 		tmp = *utmptr;
-		expand_format (tmp_str, sizeof (tmp_str), login_fmt, ai);
-		strncpy(tmp.ut_user, tmp_str, UT_NAMESIZE);
-		expand_format (tmp_str, sizeof (tmp_str), from_fmt, ai);
-		strncpy (tmp.ut_host, tmp_str, UT_HOSTSIZE);
-		tmp.ut_type = USER_PROCESS;
-		tmp.ut_time = time(NULL);
-		strncpy(tmp.ut_line, tty, UT_LINESIZE);
-		tmp.ut_addr = ai->address;
-		setutent();
-		pututline(&tmp);
-		endutent();
-		if (wtmp && (fp = fopen(WTMP_FILE, "a")) != NULL) {
-			fwrite(&tmp, sizeof(tmp), 1, fp);
-			fclose(fp);
-		}
-	}
-	else
-	{
+	} else {
+		char id[8];
 		nsyslog(LOG_ERR, "No utmp entry found when expected for line %s.", lineconf.tty);
+		snprintf(id, sizeof (id), "T%d", ai->nasport);
+	        memset(&tmp, 0, sizeof(tmp));
+		strncpy(tmp.ut_id, id, sizeof(tmp.ut_id));
+		tmp.ut_pid = getpid();
+  	}
+	
+	tty = lineconf.tty;
+	if(!strncmp(tty, "/dev/", 5))
+		tty += 5;
+	expand_format (tmp_str, sizeof (tmp_str), login_fmt, ai);
+	strncpy(tmp.ut_user, tmp_str, UT_NAMESIZE);
+	expand_format (tmp_str, sizeof (tmp_str), from_fmt, ai);
+	strncpy (tmp.ut_host, tmp_str, UT_HOSTSIZE);
+	tmp.ut_type = USER_PROCESS;
+	tmp.ut_time = time(NULL);
+	strncpy(tmp.ut_line, tty, UT_LINESIZE);
+	tmp.ut_addr = ai->address;
+	setutent();
+	pututline(&tmp);
+	endutent();
+	if (utmptr && wtmp && (fp = fopen(WTMP_FILE, "a")) != NULL) {
+		fwrite(&tmp, sizeof(tmp), 1, fp);
+		fclose(fp);
 	}
 	return 0;
 }
